@@ -535,6 +535,11 @@ public class DestinationView implements DestinationViewMBean {
         return destination.getActiveMQDestination().isDLQ();
     }
 
+    @Override
+    public void setDLQ(boolean val) {
+         destination.getActiveMQDestination().setDLQ(val);
+    }
+
     @Override
     public long getBlockedSends() {
         return destination.getDestinationStatistics().getBlockedSends().getCount();
