@@ -291,6 +291,9 @@ public class JDBCPersistenceAdapter extends DataSourceServiceSupport implements
         }
     }
 
+    @Override
+    public void allowIOResumption() {}
+
     @Override
     public void init() throws Exception {
         getAdapter().setUseExternalMessageReferences(isUseExternalMessageReferences());
@@ -484,7 +487,7 @@ public class JDBCPersistenceAdapter extends DataSourceServiceSupport implements
     }
 
     public TransactionContext getTransactionContext(ConnectionContext context) throws IOException {
-        if (context == null) {
+        if (context == null || isBrokerContext(context)) {
             return getTransactionContext();
         } else {
             TransactionContext answer = (TransactionContext)context.getLongTermStoreContext();
@@ -496,6 +499,10 @@ public class JDBCPersistenceAdapter extends DataSourceServiceSupport implements
         }
     }
 
+    private boolean isBrokerContext(ConnectionContext context) {
+        return context.getSecurityContext() != null && context.getSecurityContext().isBrokerContext();
+    }
+
     public TransactionContext getTransactionContext() throws IOException {
         TransactionContext answer = new TransactionContext(this);
         if (transactionIsolation > 0) {
