@@ -52,6 +52,7 @@ import org.apache.activemq.store.TransactionStore;
 import org.apache.activemq.store.kahadb.data.KahaLocalTransactionId;
 import org.apache.activemq.store.kahadb.data.KahaTransactionInfo;
 import org.apache.activemq.store.kahadb.data.KahaXATransactionId;
+import org.apache.activemq.store.kahadb.disk.journal.Journal.JournalDiskSyncStrategy;
 import org.apache.activemq.usage.SystemUsage;
 import org.apache.activemq.util.ServiceStopper;
 
@@ -162,6 +163,11 @@ public class KahaDBPersistenceAdapter extends LockableServiceSupport implements
         return this.letter.getLastProducerSequenceId(id);
     }
 
+    @Override
+    public void allowIOResumption() {
+        this.letter.allowIOResumption();
+    }
+
     /**
      * @param destination
      * @see org.apache.activemq.store.PersistenceAdapter#removeQueueMessageStore(org.apache.activemq.command.ActiveMQQueue)
@@ -214,7 +220,7 @@ public class KahaDBPersistenceAdapter extends LockableServiceSupport implements
      */
     @Override
     public long size() {
-        return this.letter.size();
+        return this.letter.isStarted() ? this.letter.size() : 0l;
     }
 
     /**
@@ -436,7 +442,7 @@ public class KahaDBPersistenceAdapter extends LockableServiceSupport implements
 
     /**
      * Get the enableJournalDiskSyncs
-     * @deprecated use {@link #setEnableJournalDiskSyncs} instead
+     * @deprecated use {@link #getJournalDiskSyncStrategy} instead
      * @return the enableJournalDiskSyncs
      */
     public boolean isEnableJournalDiskSyncs() {
@@ -446,7 +452,7 @@ public class KahaDBPersistenceAdapter extends LockableServiceSupport implements
     /**
      * Set the enableJournalDiskSyncs
      *
-     * @deprecated use {@link #setEnableJournalDiskSyncs} instead
+     * @deprecated use {@link #setJournalDiskSyncStrategy} instead
      * @param enableJournalDiskSyncs
      *            the enableJournalDiskSyncs to set
      */
@@ -461,6 +467,10 @@ public class KahaDBPersistenceAdapter extends LockableServiceSupport implements
         return letter.getJournalDiskSyncStrategy();
     }
 
+    public JournalDiskSyncStrategy getJournalDiskSyncStrategyEnum() {
+        return letter.getJournalDiskSyncStrategyEnum();
+    }
+
     /**
      * @param journalDiskSyncStrategy
      */
@@ -779,7 +789,7 @@ public class KahaDBPersistenceAdapter extends LockableServiceSupport implements
     @Override
     public String toString() {
         String path = getDirectory() != null ? getDirectory().getAbsolutePath() : "DIRECTORY_NOT_SET";
-        return "KahaDBPersistenceAdapter[" + path + "]";
+        return "KahaDBPersistenceAdapter[" + path + (getIndexDirectory() != null ? ",Index:" + getIndexDirectory().getAbsolutePath() : "") +  "]";
     }
 
     @Override
