@@ -56,6 +56,7 @@ import org.apache.activemq.store.TransactionIdTransformer;
 import org.apache.activemq.store.TransactionIdTransformerAware;
 import org.apache.activemq.store.TransactionStore;
 import org.apache.activemq.store.kahadb.scheduler.JobSchedulerStoreImpl;
+import org.apache.activemq.usage.StoreUsage;
 import org.apache.activemq.usage.SystemUsage;
 import org.apache.activemq.util.IOExceptionSupport;
 import org.apache.activemq.util.IOHelper;
@@ -64,6 +65,8 @@ import org.apache.activemq.util.ServiceStopper;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import static org.apache.activemq.store.kahadb.MessageDatabase.DEFAULT_DIRECTORY;
+
 /**
  * An implementation of {@link org.apache.activemq.store.PersistenceAdapter}  that supports
  * distribution of destinations across multiple kahaDB persistence adapters
@@ -150,7 +153,7 @@ public class MultiKahaDBPersistenceAdapter extends LockableServiceSupport implem
         destinationMap.setEntries(entries);
     }
 
-    private String nameFromDestinationFilter(ActiveMQDestination destination) {
+    public static String nameFromDestinationFilter(ActiveMQDestination destination) {
         if (destination.getQualifiedName().length() > IOHelper.getMaxFileNameLength()) {
             LOG.warn("Destination name is longer than 'MaximumFileNameLength' system property, " +
                      "potential problem with recovery can result from name truncation.");
@@ -242,6 +245,20 @@ public class MultiKahaDBPersistenceAdapter extends LockableServiceSupport implem
         }
         transactionStore.deleteAllMessages();
         IOHelper.deleteChildren(getDirectory());
+        for (Object o : destinationMap.get(new AnyDestination(new ActiveMQDestination[]{new ActiveMQQueue(">"), new ActiveMQTopic(">")}))) {
+            if (o instanceof FilteredKahaDBPersistenceAdapter) {
+                FilteredKahaDBPersistenceAdapter filteredKahaDBPersistenceAdapter = (FilteredKahaDBPersistenceAdapter) o;
+                if (filteredKahaDBPersistenceAdapter.getPersistenceAdapter().getDirectory() != DEFAULT_DIRECTORY) {
+                    IOHelper.deleteChildren(filteredKahaDBPersistenceAdapter.getPersistenceAdapter().getDirectory());
+                }
+                if (filteredKahaDBPersistenceAdapter.getPersistenceAdapter() instanceof KahaDBPersistenceAdapter) {
+                    KahaDBPersistenceAdapter kahaDBPersistenceAdapter = (KahaDBPersistenceAdapter) filteredKahaDBPersistenceAdapter.getPersistenceAdapter();
+                    if (kahaDBPersistenceAdapter.getIndexDirectory() != null) {
+                        IOHelper.deleteChildren(kahaDBPersistenceAdapter.getIndexDirectory());
+                    }
+                }
+            }
+        }
     }
 
     @Override
@@ -271,6 +288,13 @@ public class MultiKahaDBPersistenceAdapter extends LockableServiceSupport implem
         return maxId;
     }
 
+    @Override
+    public void allowIOResumption() {
+        for (PersistenceAdapter persistenceAdapter : adapters) {
+            persistenceAdapter.allowIOResumption();
+        }
+    }
+
     @Override
     public void removeQueueMessageStore(ActiveMQQueue destination) {
         PersistenceAdapter adapter = null;
@@ -375,31 +399,47 @@ public class MultiKahaDBPersistenceAdapter extends LockableServiceSupport implem
     }
 
     private void registerExistingAdapter(FilteredKahaDBPersistenceAdapter filteredAdapter, File candidate) throws IOException {
-        PersistenceAdapter adapter = adapterFromTemplate(filteredAdapter.getPersistenceAdapter(), candidate.getName());
+        PersistenceAdapter adapter = adapterFromTemplate(filteredAdapter, candidate.getName());
         startAdapter(adapter, candidate.getName());
         Set<ActiveMQDestination> destinations = adapter.getDestinations();
         if (destinations.size() != 0) {
-            registerAdapter(adapter, destinations.toArray(new ActiveMQDestination[]{})[0]);
+            registerAdapter(filteredAdapter, adapter, destinations.toArray(new ActiveMQDestination[]{})[0]);
         } else {
             stopAdapter(adapter, candidate.getName());
         }
     }
 
     private FilteredKahaDBPersistenceAdapter addAdapter(FilteredKahaDBPersistenceAdapter filteredAdapter, ActiveMQDestination destination) throws IOException {
-        PersistenceAdapter adapter = adapterFromTemplate(filteredAdapter.getPersistenceAdapter(), nameFromDestinationFilter(destination));
-        return registerAdapter(adapter, destination);
+        PersistenceAdapter adapter = adapterFromTemplate(filteredAdapter, nameFromDestinationFilter(destination));
+        return registerAdapter(filteredAdapter, adapter, destination);
     }
 
-    private PersistenceAdapter adapterFromTemplate(PersistenceAdapter template, String destinationName) throws IOException {
-        PersistenceAdapter adapter = kahaDBFromTemplate(template);
+    private PersistenceAdapter adapterFromTemplate(FilteredKahaDBPersistenceAdapter template, String destinationName) throws IOException {
+        PersistenceAdapter adapter = kahaDBFromTemplate(template.getPersistenceAdapter());
         configureAdapter(adapter);
         configureDirectory(adapter, destinationName);
+        configureIndexDirectory(adapter, template.getPersistenceAdapter(), destinationName);
         return adapter;
     }
 
+    private void configureIndexDirectory(PersistenceAdapter adapter, PersistenceAdapter template, String destinationName) {
+        if (template instanceof KahaDBPersistenceAdapter) {
+            KahaDBPersistenceAdapter kahaDBPersistenceAdapter = (KahaDBPersistenceAdapter) template;
+            if (kahaDBPersistenceAdapter.getIndexDirectory() != null) {
+                if (adapter instanceof KahaDBPersistenceAdapter) {
+                    File directory = kahaDBPersistenceAdapter.getIndexDirectory();
+                    if (destinationName != null) {
+                        directory = new File(directory, destinationName);
+                    }
+                    ((KahaDBPersistenceAdapter)adapter).setIndexDirectory(directory);
+                }
+            }
+        }
+    }
+
     private void configureDirectory(PersistenceAdapter adapter, String fileName) {
         File directory = null;
-        File defaultDir = MessageDatabase.DEFAULT_DIRECTORY;
+        File defaultDir = DEFAULT_DIRECTORY;
         try {
             defaultDir = adapter.getClass().newInstance().getDirectory();
         } catch (Exception e) {
@@ -417,9 +457,9 @@ public class MultiKahaDBPersistenceAdapter extends LockableServiceSupport implem
         adapter.setDirectory(directory);
     }
 
-    private FilteredKahaDBPersistenceAdapter registerAdapter(PersistenceAdapter adapter, ActiveMQDestination destination) {
+    private FilteredKahaDBPersistenceAdapter registerAdapter(FilteredKahaDBPersistenceAdapter template, PersistenceAdapter adapter, ActiveMQDestination destination) {
         adapters.add(adapter);
-        FilteredKahaDBPersistenceAdapter result = new FilteredKahaDBPersistenceAdapter(destination, adapter);
+        FilteredKahaDBPersistenceAdapter result = new FilteredKahaDBPersistenceAdapter(template, destination, adapter);
         destinationMap.put(destination, result);
         return result;
     }
