@@ -16,6 +16,11 @@
  */
 package org.apache.activemq.karaf.itest;
 
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertTrue;
+import static org.ops4j.pax.exam.karaf.options.KarafDistributionOption.editConfigurationFilePut;
+import static org.ops4j.pax.exam.karaf.options.KarafDistributionOption.replaceConfigurationFile;
+
 import java.io.File;
 import java.util.concurrent.Callable;
 
@@ -23,69 +28,41 @@ import org.junit.Ignore;
 import org.junit.Test;
 import org.junit.runner.RunWith;
 import org.ops4j.pax.exam.Configuration;
-import org.ops4j.pax.exam.MavenUtils;
 import org.ops4j.pax.exam.Option;
 import org.ops4j.pax.exam.junit.PaxExam;
 
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertTrue;
-import static org.ops4j.pax.exam.karaf.options.KarafDistributionOption.editConfigurationFilePut;
-import static org.ops4j.pax.exam.karaf.options.KarafDistributionOption.replaceConfigurationFile;
-
-//@RunWith(PaxExam.class)
-@Ignore
-public class ActiveMQBrokerNdExternalCamelFeatureTest extends AbstractJmsFeatureTest {
+@RunWith(PaxExam.class)
+public class ActiveMQBrokerNdExternalCamelFeatureTest extends AbstractFeatureTest {
 
     @Configuration
     public static Option[] configure() {
-        Option[] baseOptions = configure("activemq");
-        // copy camel.xml into a temporary directory in karaf, so we later can hot-deploy it
-        baseOptions = append(replaceConfigurationFile("data/tmp/camel.xml", new File(basedir + "/src/test/resources/org/apache/activemq/karaf/itest/camel.xml")), baseOptions);
-        baseOptions = append(editConfigurationFilePut("etc/system.properties", "camel.version", MavenUtils.getArtifactVersion("org.apache.camel.karaf", "apache-camel")), baseOptions);
-        return configureBrokerStart(baseOptions);
+        return new Option[] //
+        {
+         configure("activemq"),
+         // copy camel.xml into a temporary directory in karaf, so we later can hot-deploy it
+         replaceConfigurationFile("data/tmp/camel.xml", new File(RESOURCE_BASE + "camel.xml")),
+         editConfigurationFilePut("etc/system.properties", "camel.version", camelVersion())
+        };
     }
 
     @Ignore("camel.xml from auto deploy directory does not seem to get picked up, no idea why atm")
     @Test(timeout = 2 * 60 * 1000)
     public void test() throws Throwable {
-
         assertFeatureInstalled("activemq");
-        executeCommand("feature:repo-add " + getCamelFeatureUrl());
         installAndAssertFeature("camel");
         installAndAssertFeature("activemq-camel");
 
-        withinReason(new Callable<Boolean>() {
+        assertBrokerStarted();
+        withinReason(new Runnable() {
             @Override
-            public Boolean call() throws Exception {
-                assertEquals("brokerName = amq-broker", executeCommand("activemq:list").trim());
-                return true;
+            public void run() {
+                getBundle("org.apache.activemq.activemq-camel");
             }
         });
 
-
-        withinReason(new Callable<Boolean>(){
-            @Override
-            public Boolean call() throws Exception {
-                assertTrue(executeCommand("activemq:bstat").trim().contains("BrokerName = amq-broker"));
-                return true;
-            }
-        });
-
-        withinReason(new Callable<Boolean>() {
-            @Override
-            public Boolean call() throws Exception {
-                assertTrue("activemq-camel bundle installed", verifyBundleInstalled("org.apache.activemq.activemq-camel"));
-                return true;
-            }
-        });
-
-        assertFeatureInstalled("activemq-camel");
-
-        System.err.println(executeCommand("activemq:bstat").trim());
-
         // hot deploy the camel.xml file by copying it to the deploy directory
-        System.err.println("Karaf is running in dir: " + System.getProperty("karaf.base"));
         String karafDir = System.getProperty("karaf.base");
+        System.err.println("Karaf is running in dir: " + karafDir);
         System.err.println("Hot deploying Camel application");
         copyFile(new File(karafDir + "/data/tmp/camel.xml"), new File(karafDir + "/deploy/camel.xml"));
 
@@ -98,8 +75,10 @@ public class ActiveMQBrokerNdExternalCamelFeatureTest extends AbstractJmsFeature
         });
 
         // produce and consume
-        produceMessage("camel_in");
-        assertEquals("got our message", "camel_in", consumeMessage("camel_out"));
+        JMSTester tester = new JMSTester();
+        tester.produceMessage("camel_in");
+        assertEquals("got our message", "camel_in", tester.consumeMessage("camel_out"));
+        tester.close();
     }
 
 }
